import os, shutil, unittest, uuid, tempfile
from .decorator import lfstest


class LfsTestBase(unittest.TestCase):
    def setUp(self):
        # set up temp test directory (sandboxed for each unit test)
        self.testdir = os.path.join(tempfile.gettempdir(),
                                    str(uuid.uuid4()).replace('-', ''))
        if not os.path.exists(self.testdir):
            os.makedirs(self.testdir)

    def tearDown(self):
        # remove temp test directory
        if os.path.exists(self.testdir):
            shutil.rmtree(self.testdir)

    def get_file(self, abs_datadir, rel_src, rel_dest=None):
        if rel_dest is None or rel_dest == '':
            rel_dest = os.path.split(rel_src)[1]

        abs_src = os.path.join(abs_datadir, rel_src)
        abs_dest = os.path.join(self.testdir, rel_dest)
        lfstest().lfs_pull(abs_src, rel_src)

        dir, ext = os.path.splitext(abs_dest)
        if len(ext) > 0:
            dir = os.path.split(abs_dest)[0]
            if not os.path.exists(dir):
                os.makedirs(dir)
        shutil.copy(abs_src, abs_dest)
        return abs_dest



from functools import wraps
from nose.plugins.attrib import attr

import os
import pkg_resources
import shlex
import subprocess


class lfstest(object):
    def __init__(self, file_name=None, data_dir=None, test_type=None,
                 *args, **kwargs):
        self.named_attrib = kwargs
        self.named_attrib['type'] = test_type
        self.attrib = args
        self.file_name = file_name
        self.data_dir = data_dir

    def __call__(self, f):
        """
        If there are decorator arguments, __call__() is only called
        once, as part of the decoration process! You can only give
        it a single argument, which is the function object.
        """
        mod = f.__module__
        if self.file_name is not None:
            if isinstance(self.file_name, list):
                output = []
                for file_n in self.file_name:
                    output.append(
                        self._get_file(file_n, self.data_dir,
                                       self.named_attrib['type'], mod))
                self.file_name = output
            else:
                self.file_name = self._get_file(
                    self.file_name, self.data_dir,
                    self.named_attrib['type'], mod)
            file_name_pass = self.file_name
        else:
            file_name_pass = None

        @attr(*self.attrib, **self.named_attrib)
        @wraps(f)
        def wrapper(self, *args, **kwargs):
            if file_name_pass is None:
                f(self, *args, **kwargs)
            else:
                f(self, file_name_pass, *args, **kwargs)

        return wrapper

    def _get_file(self, file_name, data_dir, test_type, mod):
        if data_dir is None:
            data_dir = ''
        if test_type is None:
            test_type = ''
        file_name_relative = os.path.join(data_dir, test_type, file_name)
        file_name = pkg_resources.resource_filename(
            mod, file_name_relative
        )

        return self.lfs_pull(file_name, file_name_relative)

    def lfs_pull(self, file_name, file_name_relative):
        if not os.path.isfile(file_name):
            raise IOError('Filename or Pointer ({}) is not available.'.format(
                file_name))
        elif os.path.getsize(file_name) < 300:
            with open(file_name) as open_file:
                check_words = [next(open_file).split(' ')[0] for x in range(3)]
            if all(word in check_words for word in ['version', 'size', 'oid']):
                process = subprocess.Popen(
                    shlex.split(
                        'git lfs pull --include="{}" --exclude=""'.format(
                            os.path.join('**', file_name_relative))),
                    stdout=subprocess.PIPE
                )
                output, error = process.communicate()
                if error is not None:
                    raise Exception(
                        'Tried LFS pull. Failed with: {}'.format(error)
                    )
        return file_name



from setuptools import setup, find_packages
with open('noselfs/meta.py') as f:
    exec(f.read())

setup(
    name='noselfs',
    version=__version__,
    packages=find_packages(exclude=['tests']),
    description='A nosetest add-on that gets lfs files',
    author='Grant Stephens',
    author_email='grant@stephens.co.za',
    scripts=[],
    install_requires=[
        'nose',
    ],
    license='MIT',
    url='https://github.com/grantstephens/noselfs',
    download_url='https://github.com/grantstephens/noselfs/tarball/%s'
        % (__version__, ),
    keywords='nose test lfs',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    extras_require={
        'test':  ['requests-mock>=0.7.0', 'nose'],
        }
)



__version__ = "0.0.8"

"""
Changelog:
0.0.8: Updated the testbase class to handle relative paths correctly
0.0.7: Testbase remote fix
0.0.6: Versioning weirdness
0.0.5: Fix rel path error
0.0.4: Add Base test class for sandboxed tests
0.0.3: Change to the way lfs looks for files- has a greater chance of finding
    them now.
0.0.2: Change order of arguments- allows users to put they tests in more
    places, i.e.doesn't restrict to data/'type' anymore
0.0.1: Initial version. Probably buggy
"""
